"""Tasks to download, compile and install Python Imaging Library."""

from buildit.task import Task
from topp.build.lib.commands import DistUtilInstaller

install = Task(
    'Install Python Imaging Library',
    namespaces='pil',
    workdir='${srcdir}',
    targets='${deploydir}/bin/pilconvert.py',
    commands=[ DistUtilInstaller('${./download_url}') ],
    )




"""Tasks to download, configure, compile and install Zope."""

import os

join = os.path.join

from buildit.task import Task
from topp.build.lib.taskfuncs import copyskeleton
from buildit.commandlib import Patch

checkout = Task(
    'Checkout Zope',
    namespaces='zope',
    workdir='${srcdir}',
    targets=['${zope_source}'],
    commands = ['svn co ${./svn_url} ${./package}']
    )

resourcedir = join('${moduledir}', 'resources')

patch_zope = Task(
    'Patch Zope',
    namespaces='zope',
    workdir='${zope_source}',
    targets=['${zope_source}/test.py.origorig',
             '${zope_source}/lib/python/AccessControl/__init__.py.origorig',
             '${zope_source}/lib/python/Products/__init__.py.origorig',
             ],
    commands = [Patch("test.py", join(resourcedir, 'test.py.diff'), '0'),
                Patch("lib/python/Products/__init__.py", 
                      join(resourcedir, 'Products.diff'), '0'),
                Patch("lib/python/AccessControl/__init__.py", 
                      join(resourcedir, 'accontrol.diff'), '0'),
                ],
    dependencies=(checkout,)
    )

compile_zope = Task(
    'Compile Zope',
    namespaces='zope',
    targets=['${zope_source}/bin',],
    workdir='${zope_source}',
    commands=['./configure --prefix=${zope_source}',
              'make',
              'make inplace',
              ],
    dependencies=(patch_zope,)
    )

mkzeo = Task(
    'Install a zeo instance',
    namespaces='zope',
    targets='${zeo_instance}/bin/zeoctl',
    workdir='${zope_source}',
    commands=[ 'bin/mkzeoinstance.py ${zeo_instance} ${zeo_port}',],
    dependencies=(compile_zope,)
    )

copy_skel = copyskeleton(fromdir='${moduledir}/skel/zope',
                         todir='${zope_source}/custom_skel',
                         name='copy zope skeleton files',
                         namespaces='zope'
                         )

mkzope = Task(
    'Install a zope instance',
#    namespaces='zope',
    targets=['${zope_instance}/bin/runzope', '${zope_instance}/bin/zopectl'],
    workdir='${zope_source}',
    commands=[
        'bin/mkzopeinstance.py --dir ${zope_instance} --user ${zope_user}:${zope_password} --skelsrc ${zope_source}/custom_skel',
        ],
    dependencies=(compile_zope, copy_skel),
    )

linkzope = Task(
    'link zope home to zope source',
    targets='${zope_home}',
    commands=[ 'ln -s ${zope_source} ${zope_home}' ],
    dependencies=(checkout,)
    )

linkzopebinaries = Task(
    "link zope binaries to be in the env's PATH",
    targets=['${deploydir}/bin/zopectl', '${deploydir}/bin/zeoctl'],
    commands=['ln -s ${deploydir}/zope/bin/zopectl ${deploydir}/bin/',
              'ln -s ${deploydir}/zeo/bin/zeoctl ${deploydir}/bin/'],
    dependencies=(mkzope, mkzeo)
    )



""" options for this script -- topp.build.opencore """

def add_options(parser):

    ### add generic options
    
    parser.add_option('--create-site',
                      dest="create_site",
                      action="store_true",
                      default=False,
                      help="create openplans site and customizations")

    ### add options that override .ini file variables

    parser.add_context_option('--zope-port', 
                              dest='zope_port', 
                              help='Zope HTTP server port number.',
                              metavar='ZOPE-PORT'
                              )
    
    parser.add_context_option('--zeo-port', 
                              dest='zeo_port',
                              help='The port the ZEO server will listen on.', 
                              metavar="ZEO-PORT")

    parser.add_context_option('--debug-mode',
                              dest='debug_mode',
                              default='${zope/debug_mode}',
                              help="Run Zope in debug mode? (on/off, default defined by the distribution settings)",
                              metavar="DEBUG-MODE"
                              )

    parser.add_context_option('--with-zope',
                              dest='zope_source',
                              directory=True,
                              default='${srcdir}/${zope/package}',
                              help="Location of an existing Zope source checkout (must have write access?)",
                              metavar="/path/to/Zope-source-checkout",
                              )

    parser.add_context_option('--with-products',
                              dest='products',
                              directory=True,
                              default='${deploydir}/src/${opencore/package}',
                              help="Location of an existing Products bundle (will be symlinked in)",
                              metavar="/path/to/products-bundle-checkout",
                              )
    
    parser.add_context_option('--with-python',
                              dest='python',
                              help="Location of the python you want to use",
                              metavar="/path/to/python",
                              )




import os
import sys

from topp.build.lib import TOPPbuild
from topp.utils.error import error
from topp.build.opencore import options
from topp.build.opencore.tasks import pil, opencore, zope

### global variables

help = """Constructs a ready-to-run OpenCore instance from
scratch, installing Zope, Plone, and all other dependencies.  
"""

app = 'opencore'

def create_builder():
    # check python version
    if not sys.hexversion >= 33817584:
        error("this build requires at least python 2.4.3. please update.")

    # options for workingenv
    workingenv_opts = ('--home', '-r', '${opencore/requirements}' )

    # create the builder with the distributions available
        
    builder = TOPPbuild(app, 
                        workingenv_opts=workingenv_opts,
                        distributions=('local', 'live'),
                        help=help)

    # add command line options
    options.add_options(builder)

    # PIL for graphics
    builder.add_task(pil.install, name='PIL', help="don't install PIL")

    # parse options
    builder.parse_args()

    # test to make sure workingenv --home was used
    wenv = os.environ.get('WORKING_ENV')
    if wenv and builder.options.in_place:
        if not os.path.exists(os.path.join(wenv, 'lib', 'python')):
            error("workingenv must be created with --home")        

    # set variables
    builder.set_variable('zope_instance', 
                         '${deploydir}/zope') # INSTANCE_HOME
    builder.set_variable('zeo_instance', '${deploydir}/zeo')
    builder.set_variable('zope_home', '${deploydir}/lib/zope')

    # Whether to send email on login/join
    EMAILCONFIRMATION = 'True'
    if builder.options.dist == 'local':
        # configure based on whether an SMTP server is running
        from smtplib import SMTP
        import socket
        try:
            SMTP('localhost')
        except socket.error:
            EMAILCONFIRMATION = 'False'
    builder.set_variable('emailconfirmation', EMAILCONFIRMATION)

    # Zope / Plone
    builder += zope.mkzope
    builder += zope.mkzeo
    builder += zope.linkzope
    builder += zope.linkzopebinaries

    # opencore
    if builder.options.create_site:
        builder += opencore.create_site
        builder.set_final_message(os.path.join('${moduledir}', 'results-site-created.txt'))
    else:
        builder += opencore.install
        builder.set_final_message(os.path.join('${moduledir}', 'results-installed.txt'))

    return builder

def main():
    builder = create_builder()
    builder.install()



from setuptools import setup, find_packages
import sys, os

version = '0.5.1'

# add a dav client to update dep links?

setup(name='topp.build.opencore',
      version=version,
      description="buildit builder for the opencore softwar",
      long_description="""\
""",
      classifiers=[], # Get strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      keywords='build topp openplans',
      author='whit',
      author_email='jhammel@openplans.org',
      url='',
      license='GPL',
      packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
      namespace_packages=['topp', 'topp.build'],
      include_package_data=True,
      zip_safe=False,
      install_requires=['topp.build.lib>=0.5'], 
      entry_points="""
      # -*- Entry points: -*-
      [console_scripts]
      build-opencore = topp.build.opencore:main
      """,
      dependency_links=["http://www.openplans.org/projects/opencore/dependencies"],
      )
      



"""Tasks to download, configure, compile and install OpenCore."""

import os
join = os.path.join

from buildit.task import Task
from topp.build.lib.taskfuncs import copyskeleton
from buildit.commandlib import Substitute, InFileWriter

from topp.build.lib.commands import EasyInstall, TarballExtractor

# Products 
# these should be done in a better way, more svn-friendly
def products(prepend=''):
    """
    ATContentTypes            DataGridField                
    ATReferenceBrowserWidget  DocFinderTab                 ResourceRegistries
    AddRemoveWidget           ExtendedPathIndex            RichDocument
    ExternalEditor               SecureMailHost
    Fate                         TeamSpace
    Anonymailer               Five                         ZopeVersionControl
    Archetypes                GenericSetup                 contentmigration
    CMFActionIcons            GroupUserFolder              
    CMFCalendar               MailBoxer                    
    CMFCore                   MaildropHost                 i18ntestcase
    CMFDefault                Marshall                     intelligenttext
    CMFDiffTool               MimetypesRegistry            kupu
    OFolder
    CMFDynamicViewFTI         PageCacheManager             listen
    CMFEditions               PasswordResetTool            membrane
    CMFFormController         PlacelessTranslationService  
    CMFPlacefulWorkflow       PloneErrorReporting          ploneundelete
    CMFPlone                  PloneLanguageTool            remember
    CMFQuickInstallerTool     PlonePAS                     statusmessages
    CMFSetup                  PloneTestCase                testing
    CMFSquidTool              PloneTranslations            txtfilter
    CMFTestCase               PluggableAuthService         
    CMFTopic                  PluginRegistry               validation
    CMFUid                    Poi                          wicked
    CacheSetup                PolicyHTTPCacheManager
    DCWorkflow                PortalTransforms
    """
    return [ join(prepend, i) for i in products.__doc__.split() ]

bin_skel = copyskeleton(name='copy bin/ skeleton',
                        fromdir='${moduledir}/skel/bin',
                        todir='${deploydir}/bin',
                        targets=['${deploydir}/bin/accept_certificates.sh', 
                                 '${deploydir}/bin/pollzope.sh']
                        )

checkout = Task(
    'Checkout OpenPlans bundle from subversion',
    namespaces='opencore',
    targets = products('${products}'),
    commands=[ 
        '${deploydir}/bin/accept_certificates.sh',        
        'svn co ${./repository_url_bundle} ${products}', ],
    dependencies=(bin_skel,)
    )

install_bundle = Task(
    'Install Opencore bundle',
    targets = products('${zope_instance}/Products'),
    commands=[ 'ln -fs ${products}/* ${zope_instance}/Products/', ],
    dependencies=(checkout,),
    )

admin_file = Task(
    'Creating TOPP_ADMIN_INFO',
    namespaces='universal',
    targets = '${./topp_admin_info_filename}',
    commands = [ 'mkdir -p `dirname ${./topp_admin_info_filename}`',
                 'echo ${zope_user}:${zope_password} > ${./topp_admin_info_filename}',
                 'chmod 660 ${./topp_admin_info_filename}' ],
    )

checkout_xinha = Task(
    'Checkout xinha',
    workdir='${deploydir}/src/opencore/opencore/nui',
    commands=['svn co http://svn.xinha.webfactional.com/tags/0.92beta xinha'],
    dependencies=(install_bundle,),
    targets='${deploydir}/src/opencore/opencore/nui/xinha',
    )

install = Task(
    'Install OpenCore',
    namespaces=['opencore', 'zope'],
    workdir='${deploydir}',
    commands=['touch ${deploydir}/.workingenv/opencore-products-installed.txt'],
    targets=['${zope_instance}/Products',
             '${deploydir}/.workingenv/opencore-products-installed.txt'
             ],
    dependencies=(install_bundle, checkout_xinha, admin_file)
    )

runzeo = Task(
    'start zeo',
    workdir='${zeo_instance}',
    commands=['bin/zeoctl start'],
    dependencies=(install,),
    )

runzope = Task(
    'start zope',
    workdir='${zope_instance}',
    commands=['bin/zopectl fg &> log/firstrun.log & echo -n $! > var/firstrun.pid',
              '${deploydir}/bin/pollzope.sh',
              'kill `cat var/firstrun.pid`',
              '! ps `cat var/firstrun.pid`',
              'bin/zopectl start'],
    dependencies=(runzeo,),
    )

create_site = Task(
    'creating OpenPlans site',
    workdir='${zope_instance}',
    commands=[ '${deploydir}/bin/create-site.sh' ],
    dependencies=(runzope, bin_skel)
    )
